/********************************************************************
*
* FeatureClass.h
*
* DESCRIPTION:
*  Feature class types for points, lines, and areas
*
********************************************************************/

#ifndef _FEATURECLASS_H_
#define _FEATURECLASS_H_

#include "GlobalMapperTypes.h"

typedef uint16 FeatureClass_t16;

#define INVALID_FEATURE_CLASS 0xFFFF

/* ----- Feature z levels ----- */
typedef uint16 ZLevel_t16; enum
{
    Z_LVL_MIN   = 0x0000,
    Z_LVL_DFLT  = 0x8000,
    Z_LVL_MAX   = 0xFFFF,

    // Z levels for areas
    Z_LVL_LAND          = Z_LVL_DFLT - 0x0800,
    Z_LVL_URBAN_AREA    = Z_LVL_DFLT - 0x0700,
    Z_LVL_PARK          = Z_LVL_DFLT - 0x0600,
    Z_LVL_SURFACE_COVER = Z_LVL_DFLT - 0x0500,
    Z_LVL_WATER         = Z_LVL_DFLT - 0x0400,
    Z_LVL_INTERTIDAL    = Z_LVL_DFLT - 0x0300,
    Z_LVL_ISOBATH       = Z_LVL_DFLT - 0x0200,
    Z_LVL_MISC_MARINE   = Z_LVL_DFLT + 0x0100,

    // Z levels for lines
    Z_LVL_CONTOURS      = Z_LVL_DFLT - 0x0100,
    Z_LVL_ROADS         = Z_LVL_DFLT + 0x0100,
    Z_LVL_POWERLINES    = Z_LVL_DFLT + 0x0110,

    // Z levels for multiple types
    Z_LVL_BOUNDARIES    = Z_LVL_DFLT + 0x0200,
    Z_LVL_MEASUREMENT   = Z_LVL_DFLT + 0x0500,

};

/* ----- Area feature classifications ----- */
typedef FeatureClass_t16 AreaFeatureClass_t16; enum
{
    AFC_MIN  = 0,
	AFC_NONE = 0,

    AFC_ISLAND           = 1,
    AFC_RESERVATION      = 2,

    // Non-vegetative cover
    AFC_SNOW_OR_GLACIER  = 3,
    AFC_SALT             = 4,
    AFC_SAND             = 5,
    AFC_ROCK             = 6,
    AFC_REEF             = 7,

    // City metropolitan area outlines
    AFC_MIN_CITY_MA       = 10,   
    AFC_CITY_LARGE        = 10,
    AFC_CITY_SMALL        = 11,
    AFC_CITY_UNKNOWN      = 12,
    AFC_MAX_CITY_MA       = 12,

    // Coverage/Quad areas
    AFC_COVERAGE         = 15,
    AFC_MAP_CATALOG_LAYER= 16,
    AFC_WATERSHED        = 17,

    // Park classifications
    AFC_MIN_PARK         = 20,
    AFC_NTNL_PARK_MJR    = 20,
    AFC_NTNL_PARK_OTHR   = 22,
    AFC_STATE_PARK_MJR   = 23,
    AFC_STATE_PARK_OTHR  = 25,
    AFC_URBAN_PARK       = 26,
    AFC_GOLF_COURSE      = 27,
    AFC_MAX_PARK         = 27,
    
    // Vegetative and non-vegetative coverage areas
    AFC_WETLAND          = 33,
    AFC_TUNDRA           = 34,
    AFC_WOODS            = 35,
    AFC_SCRUB            = 36,
    AFC_ORCHARD          = 37, // Orchard, plantation, vineyard, etc.
    AFC_CROPLAND         = 38,
    AFC_SCATTERED_TREES  = 39,

    // Water areas
    AFC_MIN_WATER           = 40,
    AFC_OCEAN               = 40,
    AFC_STILL_WATER_MJR     = 60,
    AFC_STILL_WATER_250MI   = 61,
    AFC_STILL_WATER_100MI   = 62,
    AFC_STILL_WATER_50MI    = 63,
    AFC_STILL_WATER_10MI    = 64,
    AFC_STILL_WATER_5MI     = 65,
    AFC_STILL_WATER_1MI     = 66,
    AFC_STILL_WATER_HALF    = 67,
    AFC_STILL_WATER_LT_HALF = 68,
    AFC_STILL_WATER_UNK     = 69,
    AFC_FLOW_WATER_MJR      = 70,
    AFC_FLOW_WATER_RGNL     = 71,
    AFC_FLOW_WATER_MNR      = 72,
    AFC_FLOW_WATER_INT      = 73, // Intermittent flowing water

    AFC_STILL_WATER_INT     = 75, // Intermittent lake, reservoir
    AFC_STILL_WATER_DRY     = 76, // Dry lake, reservoir
    AFC_MAX_WATER           = 76,

    // Political areas, should be drawn only in borders
    AFC_LAND_GRANT          = 77,
    AFC_COUNTY              = 78,
    AFC_STATE               = 79,
    AFC_COUNTRY             = 80,
    AFC_SURVEY_SECTION      = 81,
    AFC_COUNTY_SUBDIVISION  = 95,

    // Marine areas
    AFC_ISOBATH             = 85, // Isobath area
    AFC_ANCHORAGE           = 86, // Anchorage area
    AFC_LAND                = 87, // Land area
    AFC_INTERTIDAL_AREA     = 88, // Intertidal area
    AFC_MARINE_INFO_AREA    = 89, // Marine info area (i.e. danger, caution, dumping, etc.)
    AFC_MARINE_TRAFFIC_ZONE = 91, // Marine traffic zone

    // Other area types
    AFC_MISC_STRUCTURE      = 82, // Misc manmade structure
    AFC_MEASUREMENT         = 83, // Measurement
    AFC_ISOHEIGHT           = 84, // Iso-height area
    AFC_RANGE_RING          = 92, // Range ring
    AFC_VIEW_SHED_COVERAGE  = 93, // View Shed coverage area
    AFC_TIN_FACE            = 94, // Face polygon for TIN (triangulated irregular network)

    // Unknown area type
    AFC_UNKNOWN             = 90,

    AFC_MAX_BUILTIN         = 95,

    AFC_FIRST_CUSTOM        = 0x200, // First custom type number  

    AFC_INVALID             = INVALID_FEATURE_CLASS
};

/* ----- Line feature classifications ----- */
typedef FeatureClass_t16 LineFeatureClass_t16; enum
{
    LFC_MIN_LINE         = 0,
    LFC_NONE             = 0,
    
    // Road line features
    LFC_MIN_ROAD         =  1,
    LFC_INTERSTATE       =  1,
    LFC_MAJOR_HWY        =  2,
    LFC_STATE_HWY        =  3,
    LFC_ARTERIAL         =  4,
    LFC_COLLECTOR        =  5,
    LFC_RESIDENTIAL      =  6,
    LFC_ALLEY            =  7,
    LFC_RAMP             =  8,
    LFC_UNPAVED          = 10,
    LFC_COUNTY_ROUTE     = 12,
    LFC_MAX_ROAD         = 12,
    
    // Streams
    LFC_STREAM           = 15,
    LFC_STREAM_INT       = 16, /* INTERMITTENT STREAM */
    LFC_STREAM_DRY       = 17, /* DRY STREAM */

    // Railroad features
    LFC_MIN_RAILROAD     = 18,
    LFC_RAILROAD_CLASS1A = 18, /* CLASS 1A RAILROAD                        */
    LFC_RAILROAD_CLASS1B = 19, /* CLASS 1B RAILROAD                        */
    LFC_RAILROAD_OTHER   = 20, /* OTHER RAILROAD CLASSIFICATION            */
    LFC_MAX_RAILROAD     = 20,

    // Other line feature types
    LFC_SHORELINE        = 21,
    LFC_TRAIL            = 22,
    LFC_DATABASE         = 23,
    LFC_GRID_LINE        = 24,  /* EVERY GRID LINE                          */
    LFC_GRID_LINE_MINOR  = 25,  /* MINOR GRID LINES                         */
    LFC_GRID_LINE_MAJOR  = 26,  /* MAJOR GRID LINES                         */
    LFC_LOCAL_FERRY      = 27,
    LFC_MJR_PLTCL        = 28,  /* MAJOR ADMIN POLITICAL BOUNDARY (STATE)   */
    LFC_MNR_PLTCL        = 29,  /* MINOR ADMIN POLITICAL BOUNDARY (COUNTY)  */
    LFC_INTRN_PLTCL      = 30,  /* INTERNATIONAL POLITICAL BOUNDRARY        */
    LFC_CITY_BDRY        = 31,  /* CITY BOUNDARY                            */
    LFC_INVISIBLE_BDRY   = 32,  /* INVISIBLE BOUNDARY                       */
    LFC_POWERLINE        = 33,  /* POWERLINE                                */
    LFC_PARK_BOUNDARY    = 34,  /* NATIONAL PARK BOUNDARY                   */
    LFC_PIPELINE         = 35,  /* PIPELINE                                 */

    // Contour lines
    LFC_MIN_CONTOUR      = 36,
    LFC_CONTOUR          = 36,  /* EVERY CONTOUR LINES                      */
    LFC_CONTOUR_MINOR    = 37,  /* MINOR CONTOUR LINES                      */
    LFC_CONTOUR_MAJOR    = 38,  /* MAJOR CONTOUR LINES					    */
    LFC_CONTOUR_SUPP     = 39,  /* SUPPLEMENTARY CONTOUR LINES              */
    LFC_MAX_CONTOUR      = 39,

    // Waterways
    LFC_MIN_WATER        = 40,
    LFC_RIVER_UNKNOWN    = 40,  /* STREAM OR RIVER (UNKNOWN LENGTH)         */
    LFC_RIVER_LT_20KM    = 41,  /* STREAM OR RIVER (< 12KM )                */
    LFC_RIVER_50KM       = 42,  /* STREAM OR RIVER (12 - 50KM )             */
    LFC_RIVER_100KM      = 43,  /* STREAM OR RIVER (50 - 100KM )            */
    LFC_RIVER_250KM      = 44,  /* STREAM OR RIVER (100 - 250KM )           */
    LFC_RIVER_GT_250KM   = 45,  /* STREAM OR RIVER (> 250KM )               */
    LFC_MAX_WATER        = 45,

	// Bathymetric contour lines
    LFC_MIN_BATHY_CONTOUR= 46,
	LFC_BATHY_CONTOUR    = 46,  /* BATHYMETRIC CONTOUR LINE					*/
	LFC_BATHY_MINOR		 = 47,  /* MINOR BATHYMETRIC CONTOUR LINE		    */
	LFC_BATHY_MAJOR      = 48,  /* MAJOR BATHYMETRIC CONTOUR LINE           */
    LFC_BATHY_SUPP       = 49,  /* SUPPLEMENTARY BATHYMETRIC CONTOUR LINE   */
    LFC_MAX_BATHY_CONTOUR= 49,

    LFC_UNKNOWN          = 50,

    // Other line types
    LFC_MEASUREMENT      = 51,      // Measurement
    LFC_RUNWAY           = 52,      // Airport runway
    LFC_TRACKLOG         = 53,      // GPS Tracklog
    LFC_MARINE_ROUTE     = 54,      // Marine (recommended) route
    LFC_DIGITIZER_LINE   = 55,      // Active draw line in Digitizer Tool
    LFC_DELETED_LINE     = 56,      // Deleted line (used for style customization of deleted line features)
    LFC_SELECTED_LINE    = 57,      // Selected line (used for style customization of selected line features)
    LFC_MAX_LINE         = 57,

    LFC_FIRST_CUSTOM     = 0x200,   // First custom line type

    LFC_INVALID          = INVALID_FEATURE_CLASS
};

/* ----- Point feature classifications ----- */
typedef FeatureClass_t16 PointFeatureClass_t16; enum
{
    PFC_MIN_POINT      = 0,
    PFC_NONE           = 0,

    // City features
    PFC_MIN_CITY         =  1,
    PFC_CITY_10M         =  1,
    PFC_CITY_1M          =  4,
    PFC_CITY_500K        =  5,
    PFC_CITY_100K        =  7,
    PFC_CITY_50K         =  8,
    PFC_CITY_10K         = 10,
    PFC_CITY_LT10K       = 12,
    PFC_CITY_UNKNOWN     = 13,  /* CITY CENTER, POPULATION UNKNOWN      */
    PFC_MAX_CITY         = 13,

    // Political entities
    PFC_MIN_POLITICAL    = 20,
    PFC_COUNTRY          = 20,  /* COUNTRY                              */
    PFC_STATE            = 22,  /* STATE, PROVINCE, ECT.                */
    PFC_COUNTY           = 23,  /* COUNTY, PARISH, ECT.                 */
    PFC_ISLAND           = 24,  /* ISLAND (ANY SIZE)                    */
    PFC_LAND_GRANT       = 25,
    PFC_MAX_POLITICAL    = 25,

    // Marine point feature
    PFC_MIN_MARINE       = 30,
    PFC_ANCHORAGE        = 30,  /* ANCHORAGE                            */
    PFC_BEACON           = 31,  /* BEACON                               */
    PFC_BOTTOM_CONDITIONS= 32,  /* BOTTOM CONDITIONS                    */
    PFC_BUOY             = 33,  /* BUOY                                 */
    PFC_DANGER           = 34,  /* DANGER POINT                         */
    PFC_LIGHTHOUSE       = 35,  /* LIGHTHOUSE                           */
    PFC_SPOT_SOUNDING    = 36,  /* SPOT SOUNDING                        */
    PFC_MAX_MARINE       = 39,

    // Lidar point types
    PFC_MIN_LIDAR = 40,
    PFC_LIDAR_UNCLASSIFIED      = 40,   /* LIDAR - UNCLASSIFIED POINT   */
    PFC_LIDAR_BUILDING          = 41,   /* LIDAR - BUILDING             */
    PFC_LIDAR_GROUND            = 42,   /* LIDAR - GROUND SHOT POINT    */
    PFC_LIDAR_VEGETATION_LOW    = 43,   /* LIDAR - LOW VEGETATION       */
    PFC_LIDAR_VEGETATION_MEDIUM = 44,   /* LIDAR - MEDIUM VEGETATION    */
    PFC_LIDAR_VEGETATION_HIGH   = 45,   /* LIDAR - HIGH VEGETATION      */
    PFC_LIDAR_WATER             = 46,   /* LIDAR - WATER                */
    PFC_LIDAR_POWER_LINES       = 47,   /* LIDAR - POWER LINES          */
    PFC_LIDAR_TOWERS_OR_POLES   = 48,   /* LIDAR - TOWERS OR POLES      */
    PFC_LIDAR_MODEL_KEY_POINT   = 49,   /* LIDAR - MODEL KEY POINT (MASS POINT) */
    PFC_MAX_LIDAR = 49,

    // Misc point features
    PFC_RANGE_RING_CENTER= 55,  /* RANGE RING CENTER                    */
    PFC_STRIKE_AND_DIP   = 56,  /* STRIKE AND DIP (GEOLOGY DATA)        */
    PFC_PICTURE          = 57,  /* PICTURE LOCATION (i.e. EXIF DATA)    */
    PFC_ADDRESS_LABEL    = 58,  /* LABEL USED ON DOWNLOADED LAYERS      */
    PFC_WAYPOINT         = 59,  /* USER WAYPOINT                        */
    PFC_ARCH             = 60,  /* ARCH                                 */
    PFC_BRIDGE           = 61,  /* BRIDGE                               */
    PFC_CLIFF            = 62,  /* CLIFF                                */
    PFC_WATERFALL        = 63,  /* WATERFALL                            */
    PFC_GEYSER           = 64,  /* GEYSER                               */
    PFC_SUMMIT           = 65,  /* SUMMIT                               */
    PFC_WRECK            = 66,  /* WRECK                                */
    PFC_MARINA           = 67,  /* MARINA                               */
    PFC_WELL             = 68,  /* WELL                                 */
    PFC_TOWER            = 69,  /* TOWER                                */
    PFC_ROCK             = 70,  /* ROCK                                 */
    PFC_BOAT_RAMP        = 71,  /* BOAT RAMP                            */
    PFC_CAMPGROUND       = 72,  /* CAMPGROUND                           */
    PFC_PARK             = 73,  /* PARK                                 */
    PFC_PICNIC_AREA      = 74,  /* PICNIC AREA                          */
    PFC_1ST_AID          = 75,  /* FIRST AID STATION                    */
    PFC_MILITARY         = 76,  /* MILITARY INSTALLATION OR RESERVATION */
    PFC_LANDMARK         = 77,  /* GENERIC LANDMARK                     */
    PFC_RESTROOMS        = 78,  /* RESTROOMS                            */
    PFC_LODGING          = 79,  /* HOTEL/MOTEL/ETC.                     */
    PFC_SHOPPING         = 80,  /* SHOPPING                             */
    PFC_GOLF_COURSE      = 81,  /* GOLF COURSE                          */
    PFC_LIBRARY          = 82,  /* LIBRARY                              */
    PFC_HOSPITAL         = 83,  /* HOSPITAL                             */
    PFC_SWIMMING_AREA    = 84,  /* SWIMMING AREA                        */
    PFC_DAM              = 85,  /* DAM                                  */
    PFC_MINE             = 86,  /* MINE                                 */
    PFC_BUILDING         = 87,  /* BUILDING                             */
    PFC_BANK             = 88,  /* BANK                                 */
    PFC_AIRPORT          = 89,  /* AIRPORT                              */
    PFC_SCHOOL           = 90,  /* SCHOOL                               */
    PFC_CHURCH           = 91,  /* CHURCH                               */
    PFC_POST_OFFICE      = 92,  /* POST OFFICE                          */
    PFC_CEMETERY         = 93,  /* CEMETERY                             */
    PFC_RESTAURANT       = 94,  /* RESTAURANT/DINING                    */
    PFC_WEIGH_STATION    = 95,  /* WEIGH STATION                        */
    PFC_AMUSEMENT        = 96,  /* AMUSEMENT PARK, AREA                 */
    PFC_STADIUM          = 97,  /* STADIUM                              */
    PFC_SPOT_ELEVATION   = 98,  /* SPOT ELEVATION                       */
    PFC_STREAM_ORIGIN    = 99,  /* STREAM ORIGIN                        */

    PFC_UNKNOWN          = 100,
    PFC_MAX_POINT        = 100,

    PFC_FIRST_CUSTOM     = 0x200,   // First custom point type

    PFC_INVALID          = INVALID_FEATURE_CLASS
};

#endif // end of file FeatureClass.h